#ifndef _assistor_intel_jpeg_h
#define _assistor_intel_jpeg_h

#ifdef USE_DDRAW

#include <ijl.h>
#include <ipl.h>



class IntelJpegAssistor : public Renderer 
{
public:
    class JDecoder;
    class JRenderer;

    IntelJpegAssistor();
    ~IntelJpegAssistor();
    int command(int argc, const char*const* argv);
    
protected:
    virtual int consume(const VideoFrame* vf) ;

    JDecoder *decoder_;
    JRenderer *renderer_;
    VideoWindow *window_;

    void setupComponents();
    void destroyComponents();

    void resetQ(int inq);
    void resetSize(int inw, int inh);

    int rw_;		/* width of rendered image */
    int rh_;		/* height of rendered image */
    int inq_;		/* input quantization */
    int inw_;		/* input width */
    int inh_;		/* input height */

    //
    // The inner classes ...
    //

public:
    class JDecoder
    {
    public:
	JDecoder(IntelJpegAssistor *, JRenderer *);
	virtual ~JDecoder();

	virtual void resetQ(int inq) {};
	virtual int consume(const JpegFrame *jf) = 0;

    protected:
	IntelJpegAssistor *assistor_;

	JRenderer *renderer_;
	JPEG_CORE_PROPERTIES prop_;

    };

    class JRenderer
    {
    public:
	JRenderer(IntelJpegAssistor *);
	virtual ~JRenderer();

	//
	// Basic interface
	//   BufferLock l(r);
	//   image = r->getImage();
	//   fill image with data
	//   r->render();
	virtual IplImage *getImage() = 0;
	virtual void render() = 0;

    protected:
	IntelJpegAssistor *assistor_;
    };

    //
    // Decode to YCbCr for the overlay renderer
    //

    class JRenderer_24_Overlay;
    class JDecoder_YCbCr : public JDecoder
    {
    public:
	JDecoder_YCbCr(IntelJpegAssistor *, JRenderer_24_Overlay *);
	~JDecoder_YCbCr();
	virtual int consume(const JpegFrame *jf);
    private:
	JRenderer_24_Overlay *overlayRenderer_;	
    };

    //
    // 24-bit same-size decoding
    //

    class JDecoder_24_RGB_same : public JDecoder
    {
    public:
	JDecoder_24_RGB_same(IntelJpegAssistor *, JRenderer *);
	~JDecoder_24_RGB_same();
	virtual int consume(const JpegFrame *jf);
    };

    //
    // 24-bit zoomed decoding
    //

    class JDecoder_24_RGB_zoom : public JDecoder
    {
    public:
	JDecoder_24_RGB_zoom(IntelJpegAssistor *, JRenderer *);
	~JDecoder_24_RGB_zoom();
	virtual int consume(const JpegFrame *jf);

    protected:
	IplImage *work_;
    };

    //
    // 24-bit decimated decoding
    //

    class JDecoder_24_RGB_decimate : public JDecoder
    {
    public:
	JDecoder_24_RGB_decimate(IntelJpegAssistor *, JRenderer *);
	~JDecoder_24_RGB_decimate();
	virtual int consume(const JpegFrame *jf);
    protected:
	IplImage *work_;
    };

    //
    // 24-bit DDrawVideoImage-based rendering
    //

    class JRenderer_24_DDrawVideoImage : public JRenderer
    {
    public:
	JRenderer_24_DDrawVideoImage(IntelJpegAssistor *);
	~JRenderer_24_DDrawVideoImage();
	
	virtual IplImage *getImage();
	virtual void render();

    protected:
	DDrawVideoImage *videoImage_;
	IplImage *image_;
    };
        
    //
    // 24-bit Overlay-based rendering
    //

    class JRenderer_24_Overlay : public JRenderer, public DDrawVideoImage::WindowMotionCallback
    {
    public:
	JRenderer_24_Overlay(IntelJpegAssistor *);
	~JRenderer_24_Overlay();

	int create();
	
	virtual IplImage *getImage() { return 0; }
	virtual void render() {}

	void windowMoved();

	int lockBuffer(void **ptr, int *pitch);
	void unlockBuffer();

    protected:
	DDrawVideoImage *videoImage_;

	LPDIRECTDRAWSURFACE7 overlay_;
	LPDIRECTDRAW7 ddraw_;
	
	void computeRestrictions();
	void enforceAlignment(RECT *src, RECT *dest);
	int stretch_;
	int srcAlign_;
	int destAlign_;
    };

    friend class JDecoder;
    friend class JDecoder_24_RGB_same;
    friend class JDecoder_24_RGB_zoom;
    friend class JDecoder_24_RGB_decimate;
    friend class JDecoder_YCbCr;
    friend class JRenderer;
    friend class JRenderer_24_DDrawVideoImage;
    friend class JRenderer_24_Overlay;
};

class IntelJpegAssistorMatcher : public Matcher 
{
public:
    IntelJpegAssistorMatcher();

    TclObject* match(const char* id) ;

};

class WriteMeOnce
{
public:
    WriteMeOnce(void *data, int len, char *file)
    {
	FILE *fp = fopen(file, "wb");
	fwrite(data, len, 1, fp);
	fclose(fp);
    }
};

class WriteMeOnceRGB
{
public:
    WriteMeOnceRGB(void *data, int len, char *file)
    {
	FILE *fp = fopen(file, "wb");
	char *d = (char *) data;

	for (int i = 0; i < len; i += 4)
	{
	    fwrite(d + i, 3, 1, fp);
	}
	fclose(fp);
    }
};

#endif

#endif // _assistor_intel_jpeg_h
